pub struct ParserBuilder<'a> { /* private fields */ }Expand description
Builder for parsers.
This is used to build parsers of excel tables. Use add_tables and
add_sheets to specify which tables it needs to parse and
which sheets to find them in.
Implementations
Sourceimpl<'a> ParserBuilder<'a>
impl<'a> ParserBuilder<'a>
Sourcepub fn add_sheet(self, sheet: impl Into<String>) -> Result<Self, ParseError>
pub fn add_sheet(self, sheet: impl Into<String>) -> Result<Self, ParseError>
Adds single sheet to parser.
Sourcepub fn add_sheets(self, sheet_pattern: Regex) -> Result<Self, ParseError>
pub fn add_sheets(self, sheet_pattern: Regex) -> Result<Self, ParseError>
Adds a pattern which is expanded to matched sheet names in the workbook.
Sourcepub fn add_table(self, table: impl Into<String>) -> Result<Self, ParseError>
pub fn add_table(self, table: impl Into<String>) -> Result<Self, ParseError>
Adds single table to parser.
Sourcepub fn add_tables(self, table_pattern: Regex) -> Result<Self, ParseError>
pub fn add_tables(self, table_pattern: Regex) -> Result<Self, ParseError>
Adds a pattern which is expanded to matched table names in the workbook.
pub fn build(self) -> Parser<'a>